
# generar datos
altH<-rnorm(100, 175, 2)
altM<-rnorm(100, 168, 2)
dH<-density(altH, from=160, to=183, n=1000)
dM<-density(altM, from=160, to=183, n=1000)
mydf<-data.frame(dH$x, dM$x, dH$y, dM$y)
mydata<-data.frame(c(altM, altH), as.factor(rep(c(1,2), times=1, each=100)))
colnames(mydata)<-c("altura", "genero")
#Generar curva ROC
library(ROCR)
pred<-prediction(mydata$altura, mydata$genero)
rocplot<-performance(pred, measure = "tpr", x.measure= "fpr")
# Calcular AUC
AUC<-performance(pred, measure = "auc") 
AUCaltura<-AUC@y.values
#Calcular cutoff óptimo
perf<-performance(pred, measure = "tpr", x.measure = "tnr")
optcut<-perf@alpha.values[[1]][which.max(perf@x.values[[1]]+perf@y.values[[1]])]
minalfa<-min(perf@alpha.values[[1]])
maxalfa<-max(perf@alpha.values[[1]][perf@alpha.values[[1]]<Inf])
dfcutoff<-data.frame( round ( perf@alpha.values[[1]], 1 ),
                      perf@x.values[[1]], 
                      perf@y.values[[1]] )

xpoint<-perf@x.values[[1]][which.max(perf@x.values[[1]]+perf@y.values[[1]])] # xcoord para cutoff óptimo
ypoint<-perf@y.values[[1]][which.max(perf@x.values[[1]]+perf@y.values[[1]])] # ycoord para cutoff óptimo
xinit<-which.max(perf@x.values[[1]]+perf@y.values[[1]])
## UI
ui <- fluidPage(
  titlePanel("Curvas ROC para análisis de pruebas diagnósticas"),
  sidebarLayout(
    sidebarPanel(
      sliderInput("cutoff", "Punto de corte:", 
                  min = 1, 
                  max = 201,
                  value =xinit, step=1)
    ),
    mainPanel(plotOutput("distPlot"),
              plotOutput("RocPlot"),
              textOutput("auc"),
              textOutput("cutoff"))
  )
)


## SERVER
server <- function(input, output) {   # se ejecuta cada vez que se visita la aplicacion

  output$distPlot <- renderPlot({ # se ejecuta cada vez que se cambia un valor
    cutoff<-input$cutoff;
    #intersX<-dH$x[as.logical(abs(diff(dH$y < dM$y)))]+cutoff 
    intersX<-(cutoff*15/201)+164;
    plot(dH$x, dH$y, ylim= range(0, pmax(dH$y,dM$y)), xlim=range(dH$x), type="l");
    lines(dM$x, dM$y);
    fin<-length(dH$x[(dH$x <= intersX)]);
    polygon(c(dH$x[1:fin],dH$x[fin]), c(dH$y[1:fin],0), col="red");
    #FP<-dH$x[(dH$x > intersX)];
    polygon(c(dH$x[fin:length(dH$x)], dH$x[fin]), c(dM$y[fin:length(dH$x)], 0), col="green");
    abline(v=intersX, lwd=2);
  })
  
  output$RocPlot<- renderPlot({
    cutoff<-input$cutoff;
    plot(rocplot, colorize=TRUE);
    abline(a=0, b=1);
    posicion<-which(cutoff == dfcutoff[,1]);
    points(rocplot@x.values[[1]][cutoff],rocplot@y.values[[1]][cutoff], type="p", pch=22);
  })

  output$auc<- renderPrint( { cat( paste( "Area Under Curve: ", AUCaltura[[1]]) ) } )
  output$cutoff<- renderPrint({cat( paste ( "Best cutoff: ", optcut )) })
}


shinyApp(ui = ui, server = server)



